# Code generated by `py-sdk-gen`. DO NOT EDIT.
from .._service import Resource, AsyncResource, HeaderTypes
from .types import (
    Customer,
    PaymentInstrumentResponse,
    PersonalDetails,
)
import typing
import pydantic


class CreateCustomerBody(pydantic.BaseModel):
    """
    CreateCustomerBody is a schema definition.
    """

    customer_id: str
    """
	Unique ID of the customer.
	"""

    personal_details: typing.Optional[PersonalDetails] = None
    """
	Personal details for the customer.
	"""


class UpdateCustomerBody(pydantic.BaseModel):
    """
    UpdateCustomerBody is a schema definition.
    """

    personal_details: typing.Optional[PersonalDetails] = None
    """
	Personal details for the customer.
	"""


ListPaymentInstruments200Response = list[PaymentInstrumentResponse]
"""
ListPaymentInstruments200Response is a schema definition.
"""


class DeactivatePaymentInstrument204Response(pydantic.BaseModel):
    """
    DeactivatePaymentInstrument204Response is a schema definition.
    """


class CustomersResource(Resource):
    def __init__(self, client):
        super().__init__(client)

    def create(
        self, body: CreateCustomerBody, headers: typing.Optional[HeaderTypes] = None
    ) -> Customer:
        """
        Create a customer

        Creates a new saved customer resource which you can later manipulate and save payment instruments to.
        """
        resp = self._client.post(
            "/v0.1/customers",
            json=body,
            headers=headers,
        )
        return pydantic.TypeAdapter(Customer).validate_python(resp.json())

    def get(self, customer_id: str, headers: typing.Optional[HeaderTypes] = None) -> Customer:
        """
        Retrieve a customer

        Retrieves an identified saved customer resource through the unique `customer_id` parameter, generated upon customer creation.
        """
        resp = self._client.get(
            f"/v0.1/customers/{customer_id}",
            headers=headers,
        )
        return pydantic.TypeAdapter(Customer).validate_python(resp.json())

    def update(
        self,
        customer_id: str,
        body: UpdateCustomerBody,
        headers: typing.Optional[HeaderTypes] = None,
    ) -> Customer:
        """
        Update a customer

        Updates an identified saved customer resource's personal details.

        The request only overwrites the parameters included in the request, all other parameters will remain with theirinitially assigned values.
        """
        resp = self._client.put(
            f"/v0.1/customers/{customer_id}",
            json=body,
            headers=headers,
        )
        return pydantic.TypeAdapter(Customer).validate_python(resp.json())

    def list_payment_instruments(
        self, customer_id: str, headers: typing.Optional[HeaderTypes] = None
    ) -> ListPaymentInstruments200Response:
        """
        List payment instruments

        Lists all payment instrument resources that are saved for an identified customer.
        """
        resp = self._client.get(
            f"/v0.1/customers/{customer_id}/payment-instruments",
            headers=headers,
        )
        return pydantic.TypeAdapter(ListPaymentInstruments200Response).validate_python(resp.json())

    def deactivate_payment_instrument(
        self, customer_id: str, token: str, headers: typing.Optional[HeaderTypes] = None
    ) -> DeactivatePaymentInstrument204Response:
        """
        Deactivate a payment instrument

        Deactivates an identified card payment instrument resource for a customer.
        """
        resp = self._client.delete(
            f"/v0.1/customers/{customer_id}/payment-instruments/{token}",
            headers=headers,
        )
        return pydantic.TypeAdapter(DeactivatePaymentInstrument204Response).validate_python(
            resp.json()
        )


class AsyncCustomersResource(AsyncResource):
    def __init__(self, client):
        super().__init__(client)

    async def create(
        self, body: CreateCustomerBody, headers: typing.Optional[HeaderTypes] = None
    ) -> Customer:
        """
        Create a customer

        Creates a new saved customer resource which you can later manipulate and save payment instruments to.
        """
        resp = await self._client.post(
            "/v0.1/customers",
            json=body,
            headers=headers,
        )
        return pydantic.TypeAdapter(Customer).validate_python(resp.json())

    async def get(self, customer_id: str, headers: typing.Optional[HeaderTypes] = None) -> Customer:
        """
        Retrieve a customer

        Retrieves an identified saved customer resource through the unique `customer_id` parameter, generated upon customer creation.
        """
        resp = await self._client.get(
            f"/v0.1/customers/{customer_id}",
            headers=headers,
        )
        return pydantic.TypeAdapter(Customer).validate_python(resp.json())

    async def update(
        self,
        customer_id: str,
        body: UpdateCustomerBody,
        headers: typing.Optional[HeaderTypes] = None,
    ) -> Customer:
        """
        Update a customer

        Updates an identified saved customer resource's personal details.

        The request only overwrites the parameters included in the request, all other parameters will remain with theirinitially assigned values.
        """
        resp = await self._client.put(
            f"/v0.1/customers/{customer_id}",
            json=body,
            headers=headers,
        )
        return pydantic.TypeAdapter(Customer).validate_python(resp.json())

    async def list_payment_instruments(
        self, customer_id: str, headers: typing.Optional[HeaderTypes] = None
    ) -> ListPaymentInstruments200Response:
        """
        List payment instruments

        Lists all payment instrument resources that are saved for an identified customer.
        """
        resp = await self._client.get(
            f"/v0.1/customers/{customer_id}/payment-instruments",
            headers=headers,
        )
        return pydantic.TypeAdapter(ListPaymentInstruments200Response).validate_python(resp.json())

    async def deactivate_payment_instrument(
        self, customer_id: str, token: str, headers: typing.Optional[HeaderTypes] = None
    ) -> DeactivatePaymentInstrument204Response:
        """
        Deactivate a payment instrument

        Deactivates an identified card payment instrument resource for a customer.
        """
        resp = await self._client.delete(
            f"/v0.1/customers/{customer_id}/payment-instruments/{token}",
            headers=headers,
        )
        return pydantic.TypeAdapter(DeactivatePaymentInstrument204Response).validate_python(
            resp.json()
        )
